"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _getUserConfigPlugins = _interopRequireDefault(require("af-webpack/getUserConfigPlugins"));

var _umiUtils = require("umi-utils");

var _path = require("path");

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _browsers = _interopRequireDefault(require("../defaultConfigs/browsers"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const plugins = (0, _getUserConfigPlugins.default)();

function noop() {
  return true;
}

const excludes = ['entry', 'outputPath'];

function _default(api) {
  const debug = api.debug,
        cwd = api.cwd,
        config = api.config,
        paths = api.paths; // 把 af-webpack 的配置插件转化为 umi-build-dev 的

  api._registerConfig(() => {
    return plugins.filter(p => !excludes.includes(p.name)).map(({
      name,
      validate = noop
    }) => {
      return api => ({
        name,
        validate,

        onChange(newConfig) {
          try {
            debug(`Config ${name} changed to ${JSON.stringify(newConfig[name])}`);
          } catch (e) {}

          if (name === 'proxy') {
            global.g_umi_reloadProxy(newConfig[name]);
          } else {
            api.service.restart(`${name} changed`);
          }
        }

      });
    });
  });

  api.chainWebpackConfig(webpackConfig => {
    webpackConfig.resolve.alias.set('react', (0, _umiUtils.compatDirname)('react/package.json', cwd, (0, _path.dirname)(require.resolve('react/package.json')))).set('react-dom', (0, _umiUtils.compatDirname)('react-dom/package.json', cwd, (0, _path.dirname)(require.resolve('react-dom/package.json')))).set('react-router', (0, _umiUtils.compatDirname)('react-router/package.json', cwd, (0, _path.dirname)(require.resolve('react-router/package.json')))).set('react-router-dom', (0, _umiUtils.compatDirname)('react-router-dom/package.json', cwd, (0, _path.dirname)(require.resolve('react-router-dom/package.json')))).set('history', (0, _umiUtils.compatDirname)('umi-history/package.json', cwd, (0, _path.dirname)(require.resolve('umi-history/package.json')))).set('@', paths.absSrcPath).set('umi/link', (0, _path.join)(process.env.UMI_DIR, 'lib/link.js')).set('umi/dynamic', (0, _path.join)(process.env.UMI_DIR, 'lib/dynamic.js')).set('umi/navlink', (0, _path.join)(process.env.UMI_DIR, 'lib/navlink.js')).set('umi/redirect', (0, _path.join)(process.env.UMI_DIR, 'lib/redirect.js')).set('umi/router', (0, _path.join)(process.env.UMI_DIR, 'lib/router.js')).set('umi/withRouter', (0, _path.join)(process.env.UMI_DIR, 'lib/withRouter.js')).set('umi/_renderRoutes', (0, _path.join)(process.env.UMI_DIR, 'lib/renderRoutes.js')).set('umi/_createHistory', (0, _path.join)(process.env.UMI_DIR, 'lib/createHistory.js'));
  });
  api.modifyAFWebpackOpts(memo => {
    const browserslist = config.browserslist || _browsers.default;
    const isDev = process.env.NODE_ENV === 'development';
    const entryScript = (0, _path.join)(cwd, `./${paths.tmpDirPath}/umi.js`);
    const setPublicPathFile = (0, _path.join)(__dirname, '../../template/setPublicPath.js');
    const setPublicPath = config.runtimePublicPath || config.exportStatic && config.exportStatic.dynamicRoot;
    const entry = isDev ? {
      umi: [...(process.env.HMR === 'none' ? [] : [_reactDevUtils.webpackHotDevClientPath]), entryScript]
    } : {
      umi: [...(setPublicPath ? [setPublicPathFile] : []), entryScript]
    };
    return _objectSpread({}, memo, config, {
      cwd,
      browserslist,
      entry,
      outputPath: paths.absOutputPath,
      disableDynamicImport: true,
      babel: config.babel || {
        presets: [[require.resolve('babel-preset-umi'), {
          targets: {
            browsers: browserslist
          }
        }], ...(config.extraBabelPresets || [])],
        plugins: config.extraBabelPlugins || []
      },
      define: _objectSpread({
        'process.env.BASE_URL': config.base || '/',
        __UMI_BIGFISH_COMPAT: process.env.BIGFISH_COMPAT,
        __UMI_HTML_SUFFIX: !!(config.exportStatic && typeof config.exportStatic === 'object' && config.exportStatic.htmlSuffix)
      }, config.define || {}),
      publicPath: isDev ? '/' : config.publicPath || '/'
    });
  });
}